#include "pix.h"
#include <stdexcept>
#include <string>


/*
 *  Pix
 */

Pix::Pix(const char ** xpm)
{
	attrs_.valuemask = XpmSize | XpmExactColors | XpmCloseness;
	attrs_.exactColors = False;
	attrs_.closeness = 65535;
	const int err_code
		(X11::Inst().CreatePixmapFromData(xpm, &pixmap_,
		                                  &shapemask_, &attrs_));
	if (err_code != XpmSuccess) {
		const std::string msg ("Pixmap creation failed: ");
		throw std::runtime_error (msg + XpmGetErrorString(err_code));
	}
}

Pix::~Pix()
{
	X11::Inst().FreePixmap(pixmap_);
	X11::Inst().FreePixmap(shapemask_);
}

void Pix::Draw(Coord pos) const
{
	pos -= Coord(Width() / 2, Height() / 2);
	X11::Inst().SetClipOrigin(pos);
	X11::Inst().SetClipMask(shapemask_);
	X11::Inst().CopyAreaToWindow(pixmap_, 0, 0, Width(), Height(), pos);
	X11::Inst().SetClipMask(None);
}

void Pix::Erase(Coord pos) const
{
	pos -= Coord(Width() / 2, Height() / 2);
	X11::Inst().SetClipOrigin(pos);
	X11::Inst().SetClipMask(shapemask_);
	X11::Inst().ClearArea(pos, Width(), Height());
	X11::Inst().SetClipMask(None);
}



/*
 *  PixKeeper
 */

PixKeeper * PixKeeper::singleton_ (0);

PixKeeper::~PixKeeper()
{
	for (PixCtr::const_iterator it (pixes_.begin());
	     it != pixes_.end(); ++it) {
		delete it->second;
	}
}

Pix * PixKeeper::Get(const char ** pix_data)
{
	const PixCtr::iterator it (pixes_.find(pix_data));
	if (it != pixes_.end()) return it->second;

	Pix *const pix (new Pix(pix_data));
	pixes_.insert(PixCtr::value_type(pix_data, pix));
	return pix;
}
